unit uConfirmDialog;
{$INCLUDE 'NamedFoldersLite.inc'}
{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  Windows, SysUtils, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uLog, uMessages, uConsts, uInfo;

type
  TConfirmOverwriteDialog = class(TButtonsDialog)
  public
    type TConfirmOverwrite = (coOverwrite, coOverwriteAll, coSkip, coSkipAll, coCancel);
    type
      TItemDescription = class
      private
        fRight: string;
        fMiddle: string;
        fLeft: string;
        function GetEmpty: boolean;
      public
        procedure Clear;
        procedure ToLabel(ALabel: TFarDialog.TLabel);
        property Left: string read fLeft write fLeft;
        property Middle: string read fMiddle write fMiddle;
        property Right: string read fRight write fRight;
        property Empty: boolean read GetEmpty;
      end;
  private
    fMessage: string;
    fItem1Description: TItemDescription;
    fItem2Description: TItemDescription;
  private
    fLabelMessage: TFarDialog.TLabel;
    fLabelItem1: TFarDialog.TLabel;
    fLabelItem2: TFarDialog.TLabel;
    fLabelSeparator: TFarDialog.TSeparator;
    fCheckboxRemember: TFarDialog.TCheckbox;
    fButtonOverwrite: TFarDialog.TButton;
    fButtonSkip: TFarDialog.TButton;
    fButtonCancel: TFarDialog.TButton;
  protected
    procedure Clear; override;
    procedure CreateUserControls; override;
    procedure CreateButtonControls; override;
    procedure SetSize(Width, Height: Integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    property LabelMessage: TFarDialog.TLabel read fLabelMessage;
    property LabelItem1: TFarDialog.TLabel read fLabelItem1;
    property LabelItem2: TFarDialog.TLabel read fLabelItem2;
    property LabelSeparator: TFarDialog.TSeparator read fLabelSeparator;
    property CheckboxRemember: TFarDialog.TCheckbox read fCheckboxRemember;
    property ButtonOverwrite: TFarDialog.TButton read fButtonOverwrite;
    property ButtonSkip: TFarDialog.TButton read fButtonSkip;
    property ButtonCancel: TFarDialog.TButton read fButtonCancel;
  protected
    class function GetDescription(const Left, Middle, Right: string): TConfirmOverwriteDialog.TItemDescription;
  public
    class function ShowModal(const AMessage: string; AItem1Description, AItem2Description: TItemDescription): TConfirmOverwrite;
    constructor Create;
    destructor Destroy; override;
    function Execute: TConfirmOverwrite;
    property Message: string read fMessage write fMessage;
    property Item1Description: TItemDescription read fItem1Description write fItem1Description;
    property Item2Description: TItemDescription read fItem2Description write fItem2Description;
  end;

  TConfirmOverwriteNamedFolderDialog = class(TConfirmOverwriteDialog)
  private
  protected
    class function GetInfoDescription(Info: TNamedFolderInfo; TagMessage: TMessages): TConfirmOverwriteDialog.TItemDescription;
  public
    class function ShowModal(OldInfo, NewInfo: TNamedFolderInfo): TConfirmOverwriteDialog.TConfirmOverwrite;
  end;

  TConfirmOverwriteFileDialog = class(TConfirmOverwriteDialog)
  private
  protected
    class function GetFileDescription(const FileName: string; TagMessage: TMessages): TConfirmOverwriteDialog.TItemDescription;
  public
    class function ShowModal(const FileName: string): TConfirmOverwriteDialog.TConfirmOverwrite;
  end;

implementation

{ TConfirmOverwriteDialog }

procedure TConfirmOverwriteDialog.Clear;
begin
  inherited;
  fCheckboxRemember := nil;
  fLabelMessage := nil;
  fLabelItem1 := nil;
  fLabelItem2 := nil;
  fLabelSeparator := nil;
  fCheckboxRemember := nil;
  fButtonOverwrite := nil;
  fButtonSkip := nil;
  fButtonCancel := nil;
end;

constructor TConfirmOverwriteDialog.Create;
begin
  inherited Create(DIALOG_CONFIRMOVERWRITE_GUID);
end;

procedure TConfirmOverwriteDialog.CreateButtonControls;
begin
  inherited;
  fButtonOverwrite := AddButton(0, 0, GetMsg(MConfirmOverwriteOverwrite));
  fButtonOverwrite.Default := True;
  fButtonSkip := AddButton(0, 0, GetMsg(MConfirmOverwriteSkip));
  fButtonCancel := AddButton(0, 0, Plugin.GetCommonString(TFarPlugin.TCommonString.MCancel));
end;

procedure TConfirmOverwriteDialog.CreateUserControls;
begin
  inherited;
  fLabelMessage := AddLabel(0, 0, 0, '');
  fLabelItem1 := AddLabel(0, 0, 0, '', taRight);
  fLabelItem2 := AddLabel(0, 0, 0, '', taRight);
  fLabelSeparator := AddSeparator(0);
  fCheckboxRemember := AddCheckbox(0, 0, GetMsg(MConfirmOverwriteRemember), False);
end;

destructor TConfirmOverwriteDialog.Destroy;
begin
  inherited;
end;

function TConfirmOverwriteDialog.Execute: TConfirmOverwrite;

  procedure ShowHideLabel(ALabel: TFarDialog.TLabel; AVisible: boolean);
  begin
    if ALabel.Visible <> AVisible then
    begin
      if AVisible then
        Self.Height := Self.Height + 1
      else
        Self.Height := Self.Height - 1;
    end;
    ALabel.Visible := AVisible;
  end;

var
  Button: TFarDialog.TButton;
begin
  Result := coCancel;
  LabelMessage.Text := Message;
  ShowHideLabel(LabelItem1, (Item1Description <> nil) and (not Item1Description.Empty));
  ShowHideLabel(LabelItem2, (Item2Description <> nil) and (not Item2Description.Empty));
  Item1Description.ToLabel(LabelItem1);
  Item2Description.ToLabel(LabelItem2);
  if inherited Execute(Button) then
  begin
    if Button = ButtonOverwrite then
    begin
      if CheckboxRemember.Checked then
        Result := coOverwriteAll
      else
        Result := coOverwrite;
    end
    else if Button = ButtonSkip then
    begin
      if CheckboxRemember.Checked then
        Result := coSkipAll
      else
        Result := coSkip;
    end;
  end;
end;

class function TConfirmOverwriteDialog.GetDescription(const Left, Middle, Right: string): TConfirmOverwriteDialog.TItemDescription;
begin
  Result := TConfirmOverwriteDialog.TItemDescription.Create;
  try
    Result.Left := Left;
    Result.Middle := Middle;
    Result.Right := Right;
  except
    FreeAndNil(Result);
    Raise;
  end;
end;

procedure TConfirmOverwriteDialog.SetSize(Width, Height: Integer);
begin
  inherited;
  AlignIfVisible(LabelMessage);
  AlignIfVisible(LabelItem1);
  AlignIfVisible(LabelItem2);
  AlignIfVisible(LabelSeparator);
  AlignIfVisible(CheckboxRemember);
  CheckboxRemember.Left := LabelMessage.Left;
  AlignIfVisible(LabelButtonSeparator);
  Item1Description.ToLabel(LabelItem1);
  Item2Description.ToLabel(LabelItem2);
  ButtonOverwrite.Top := LabelButtonSeparator.Top + 1;
  ButtonOverwrite.Default := True;
  ButtonSkip.Top := ButtonOverwrite.Top;
  ButtonCancel.Top := ButtonOverwrite.Top;
end;

class function TConfirmOverwriteDialog.ShowModal(const AMessage: string; AItem1Description, AItem2Description: TItemDescription): TConfirmOverwrite;
var
  Dialog: TConfirmOverwriteDialog;
begin
  Dialog := TConfirmOverwriteDialog.Create;
  try
    Dialog.Flags := Dialog.Flags + FDLG_WARNING;
    Dialog.Message := AMessage;
    Dialog.Item1Description := AItem1Description;
    Dialog.Item2Description := AItem2Description;
    Result := Dialog.Execute;
  finally
    FreeAndNil(Dialog);
  end;
end;

procedure TConfirmOverwriteDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
begin
  inherited;
  Height := 11;
end;

{ TConfirmOverwriteNamedFolderDialog }

class function TConfirmOverwriteNamedFolderDialog.GetInfoDescription(Info: TNamedFolderInfo; TagMessage: TMessages): TConfirmOverwriteDialog.TItemDescription;
begin
  Result := GetDescription(GetMsg(TagMessage) + ' ' + Info.Name + ': ', Info.DisplayableDescription, '');
end;

class function TConfirmOverwriteNamedFolderDialog.ShowModal(OldInfo, NewInfo: TNamedFolderInfo): TConfirmOverwriteDialog.TConfirmOverwrite;
var
  Desc1, Desc2: TConfirmOverwriteDialog.TItemDescription;
begin
  Desc1 := GetInfoDescription(OldInfo, MConfirmOverwriteTagNew);
  try
    Desc2 := GetInfoDescription(NewInfo, MConfirmOverwriteTagOld);
    try
      Result := inherited ShowModal(GetMsg(MConfirmOverwriteFile), Desc1, Desc2);
    finally
      FreeAndNil(Desc2);
    end;
  finally
    FreeAndNil(Desc1);
  end;
end;

{ TConfirmOverwriteFileDialog }

class function TConfirmOverwriteFileDialog.GetFileDescription(const FileName: string; TagMessage: TMessages): TConfirmOverwriteDialog.TItemDescription;
var
  FileInfo, DateTimeFormat: string;
  SR: TSearchRec;
  {$IFNDEF DELPHIXE2_UP}
  SysTime: TSystemTime;
  {$ENDIF}
  ModificationDate: TDateTime;
begin
  if FindFirst(FileName, faAnyFile, SR) = 0 then
    try
      if (SR.Attr and faDirectory) <> 0 then
        FileInfo := GetMsg(MDirectorySize)
      else
        FileInfo := FormatCurr('###,###,###,##0', SR.Size);
      DateTimeFormat := GetMsg(MDateTimeFormat);
      if DateTimeFormat = '' then
        DateTimeFormat := 'c';
      {$IFDEF DELPHIXE2_UP}
      ModificationDate := SR.Timestamp;
      {$ELSE}
      if FileTimeToSystemTime(SR.FindData.ftLastWriteTime, SysTime) then
        ModificationDate := EncodeDate(SysTime.wYear, SysTime.wMonth, SysTime.wDay) + EncodeTime(SysTime.wHour, SysTime.wMinute, SysTime.wSecond, SysTime.wMilliseconds)
      else
        ModificationDate := 0;
      {$ENDIF}
      FileInfo := FileInfo + ' ' + FormatDateTime(DateTimeFormat, ModificationDate);
    finally
      FindClose(SR);
    end
  else
    FileInfo := GetMsg(MFailedToReadFileInfo);
  Result := GetDescription(GetMsg(TagMessage) + ' ', FileName, ' ' + FileInfo);
end;

class function TConfirmOverwriteFileDialog.ShowModal(const FileName: string): TConfirmOverwriteDialog.TConfirmOverwrite;
var
  Desc2: TConfirmOverwriteDialog.TItemDescription;
begin
  Desc2 := GetFileDescription(FileName, MConfirmOverwriteTagOld);
  try
    Result := inherited ShowModal(GetMsg(MConfirmOverwriteFile), nil, Desc2);
  finally
    FreeAndNil(Desc2);
  end;
end;

{ TConfirmOverwriteDialog.TItemDescription }

procedure TConfirmOverwriteDialog.TItemDescription.Clear;
begin
  Left := '';
  Middle := '';
  Right := '';
end;

function TConfirmOverwriteDialog.TItemDescription.GetEmpty: boolean;
begin
  Result := (Left = '') and (Middle = '') and (Right = '');
end;

procedure TConfirmOverwriteDialog.TItemDescription.ToLabel(ALabel: TFarDialog.TLabel);
var
  MiddleWidth: integer;
begin
  if Self = nil then
  begin
    ALabel.Text := '';
  end
  else if Self.Empty then
  begin
    ALabel.Text := '';
  end
  else
  begin
    MiddleWidth := ALabel.Width - Length(Self.Left) - Length(Self.Right);
    if MiddleWidth > 0 then
      ALabel.Text := Format('%s%-*.*s%s', [Self.Left, MiddleWidth, MiddleWidth, TFarModule.TruncateString(Self.Middle, MiddleWidth, ' '), Self.Right])
    else
      ALabel.Text := Format('%s%s', [Self.Left, Self.Right]);
  end;
end;

end.
