unit uInfo;
{$INCLUDE 'NamedFoldersLite.inc'}

interface

uses
  Windows, SysUtils, Classes, Generics.Collections,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uSystem, uLog, uMessages, uConsts, uFunctions;

type
  TNamedFolderInfo = class
  private
    fName: string;
    fDirectory: string;
    fParam: string;
    fPluginID: TGUID;
    fHostFile: string;
    fDescription: string;
    fFlags: integer;
    function GetDisplayableDescription: string;
    procedure SetName(const Value: string);
    function GetIsDirectory: boolean;
    procedure SetIsDirectory(const Value: boolean);
  protected
  public
    class function NormalizeName(const Value: string): string;
    constructor Create;
    property Name: string read fName write SetName;
    property Directory: string read fDirectory write fDirectory;
    property PluginID: TGUID read fPluginID write fPluginID;
    property Param: string read fParam write fParam;
    property HostFile: string read fHostFile write fHostFile;
    property Description: string read fDescription write fDescription;
    property DisplayableDescription: string read GetDisplayableDescription;
    property Flags: integer read fFlags write fFlags;
    property IsDirectory: boolean read GetIsDirectory write SetIsDirectory;
  end;

  TNamedFolderInfoList = class(TObjectList<TNamedFolderInfo>)
  end;

implementation

{ TNamedFolderInfo }

constructor TNamedFolderInfo.Create;
begin
  inherited Create;
  PluginID := NULL_GUID;
end;

function TNamedFolderInfo.GetDisplayableDescription: string;
begin
  if Description <> '' then
    Result := Description
  else
    Result := Directory;
end;

function TNamedFolderInfo.GetIsDirectory: boolean;
begin
  Result := (Flags and FILE_ATTRIBUTE_DIRECTORY) <> 0;
end;

class function TNamedFolderInfo.NormalizeName(const Value: string): string;
begin
  Result := GetAbsoluteDirectory('', Value);
end;

procedure TNamedFolderInfo.SetIsDirectory(const Value: boolean);
begin
  if Value then
    Flags := Flags or FILE_ATTRIBUTE_DIRECTORY
  else
    Flags := Flags and (not FILE_ATTRIBUTE_DIRECTORY);
end;

procedure TNamedFolderInfo.SetName(const Value: string);
begin
  fName := NormalizeName(Value);
end;

end.
