unit uPanel;
{$INCLUDE 'NamedFoldersLite.inc'}

interface

uses
  Windows, SysUtils, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  NativeXml,
  uLog, uOptions, uMessages, uConsts, uXml, uFunctions,
  uConfirmDialog, uMakeDirDialog, uInfo;

type
  TNamedFoldersPanel = class(TFarPanelManaged)
  public
    type
      TOverwriteState = (osDefault, osAlwaysOverwrite, osAlwaysSkip);
  protected
    type
      TPanelItemInfo = record
        FileName: string;
        PanelFileName: string;
        Description: string;
        IsDirectory: boolean;
        //Flags: integer;
      end;
      TPanelItemArray = array of TPanelItemInfo;
  private
    fPanelItems: packed array of TPluginPanelItem;
    fPanelItemsHelp: TPanelItemArray;
    fPanelItemsValid: boolean;
    fDirectory: string;
    fTitle: string;
    fFileToBeSelected: string;
    fOverwriteState: TOverwriteState;
    procedure SetDirectory(const Value: string);
    procedure SaveLastDirectory;
    procedure OpenLastDirectory;
  protected
    procedure GetPanelInfo(var Info: TOpenPanelInfo); override;
    function GetTitle: string; override;
    function GetCurrentDirectory: string; override;
    function GetPanelItems(const Mode: TIntPtr; out Items: PPluginPanelItemArray; out Count: TIntPtr): boolean; override;
    function GetPanelItemsCount: integer;
    procedure FreePanelItems(const Items: PPluginPanelItemArray; const Count: TIntPtr); override;
    function MakeDirectory(const Name: string): boolean; override;
    function CanDeleteFiles(Count, OpMode: integer): boolean; override;
    function DeleteFile(const Item: PPluginPanelItem; OpMode: integer): boolean; override;
    function CanGetFiles(Count, OpMode: integer): boolean; override;
    function GetFile(const Item: PPluginPanelItem; OpMode: integer; const Dir: string; Move: boolean): boolean; override;
    function CanPutFiles(Count, OpMode: integer): boolean; override;
    function PutFile(const Item: PPluginPanelItem; OpMode: integer; const Dir: string; Move: boolean): TFarPanelManaged.TPutFileResult; override;
    function ProcessKeyEvent(Key: integer; ControlState: DWORD): boolean; override;
    function ProcessEvent(Event: integer; Param: Pointer): TIntPtr; override;
    procedure ValidatePanelItems;
    procedure ReloadPanelContents;
    function DecodeCanOverwriteResult(Res: TConfirmOverwriteDialog.TConfirmOverwrite; var OverwriteState: TOverwriteState): boolean;
    function AskCanOverwriteNamedFolder(OldInfo, NewInfo: TNamedFolderInfo; var OverwriteState: TOverwriteState): boolean;
    function AskCanOverwriteFile(const FileName: string; var OverwriteState: TOverwriteState): boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    function ChangeDirectory(const Name: string): boolean; override;
    function DeleteNamedFolder(const Name: string): boolean;
    function EditNamedFolder(const Name: string): boolean;
    function NewNamedFolder: boolean;
    function ExecuteNamedFolder(InCurrentPanel: boolean; out Handled: boolean): boolean;
    function ExportNamedFolderToDirectory(const Name, TargetDirectory: string; DeleteOriginal, Silent: boolean; var OverwriteState: TOverwriteState): boolean; overload;
    function ExportNamedFolderToDirectory(Info: TNamedFolderInfo; const TargetDirectory: string; DeleteOriginal, Silent: boolean; var OverwriteState: TOverwriteState): boolean; overload;
    function ExportNamedFolderToFile(Info: TNamedFolderInfo; const TargetFileName: string; DeleteOriginal, Silent: boolean; var OverwriteState: TOverwriteState): boolean;
    function ImportNamedFolderFile(const FileName, SourceDirectory, DestinationDirectory: string; DeleteOriginal, Silent: boolean; var OverwriteState: TOverwriteState): boolean;
    function SelectFile(const Name: string): boolean;
    property FileToBeSelected: string read fFileToBeSelected write fFileToBeSelected;
  end;

implementation

uses
  uTasks;

{ TNamedFoldersPanel }

function TNamedFoldersPanel.AskCanOverwriteFile(const FileName: string; var OverwriteState: TOverwriteState): boolean;
begin
  case OverwriteState of
    osDefault:
      Result := DecodeCanOverwriteResult(TConfirmOverwriteFileDialog.ShowModal(FileName), OverwriteState);
    osAlwaysOverwrite:
      Result := True;
    osAlwaysSkip:
      Result := False;
    else
      Result := False;
  end;
end;

function TNamedFoldersPanel.AskCanOverwriteNamedFolder(OldInfo, NewInfo: TNamedFolderInfo; var OverwriteState: TOverwriteState): boolean;
begin
  case OverwriteState of
    osDefault:
      Result := DecodeCanOverwriteResult(TConfirmOverwriteNamedFolderDialog.ShowModal(OldInfo, NewInfo), OverwriteState);
    osAlwaysOverwrite:
      Result := True;
    osAlwaysSkip:
      Result := False;
    else
      Result := False;
  end;
end;

function TNamedFoldersPanel.CanDeleteFiles(Count, OpMode: integer): boolean;
begin
  Result := True;
  if (OpMode and OPM_SILENT) = 0 then
    if ShowMessage([GetMsg(MTitle), GetMsg(MConfirmDeletion), GetMsg(MYes), GetMsg(MNo), GetMsg(MCancel)], FMSG_WARNING, 3) <> 0 then
      Result := False;
end;

function TNamedFoldersPanel.CanGetFiles(Count, OpMode: integer): boolean;
begin
  fOverwriteState := osDefault;
  Result := True;
end;

function TNamedFoldersPanel.CanPutFiles(Count, OpMode: integer): boolean;
begin
  fOverwriteState := osDefault;
  Result := True;
end;

function TNamedFoldersPanel.ChangeDirectory(const Name: string): boolean;
var
  FullName: string;
  Info: TNamedFolderInfo;
begin
  //Log('TNamedFoldersPanel.ChangeDirectory("%s")', [Name]);
  Result := False;
  if Name <> '..' then
    FullName := GetAbsoluteDirectory(Self.CurrentDirectory, '/' + Name)
  else
    FullName := GetAbsoluteDirectory(Self.CurrentDirectory, Name);
  //Log('- absolute dir "%s"', [FullName]);
  if FullName = '' then begin
    //Log('- going to root');
    SetDirectory('/');
    Result := True;
  end
  else if TNamedFolders.ReadInfoFromOptions(FullName, Info, False) then
    try
      //Log('- found info');
      if Info.IsDirectory then begin
        //Log('going to "/%s"', [FullName]);
        SetDirectory('/' + FullName);
        Result := True;
      end;
    finally
      FreeAndNil(Info);
    end
  else begin
    //Log('- directory not found');
  end;
  //Log('TNamedFoldersPanel.ChangeDirectory("%s") -> %d', [Name, Integer(Result)]);
end;

procedure TNamedFoldersPanel.Clear;
begin
  fPanelItemsValid := False;
end;

constructor TNamedFoldersPanel.Create;
begin
  inherited Create;
  OpenLastDirectory;
end;

function TNamedFoldersPanel.DecodeCanOverwriteResult(Res: TConfirmOverwriteDialog.TConfirmOverwrite; var OverwriteState: TOverwriteState): boolean;
begin
  Result := False;
  case Res of
    coOverwrite:
      begin
        Result := True;
      end;
    coOverwriteAll:
      begin
        Result := True;
        OverwriteState := osAlwaysOverwrite;
      end;
    coSkip:
      begin
        Result := False;
      end;
    coSkipAll:
      begin
        Result := False;
        OverwriteState := osAlwaysSkip;
      end;
    coCancel:
      begin
        Abort;
      end;
  end;
end;

function TNamedFoldersPanel.DeleteFile(const Item: PPluginPanelItem; OpMode: integer): boolean;
begin
  Result := DeleteNamedFolder(CopyString(Item^. {$IFDEF FAR3_UP} FileName {$ELSE} FindData.cFileName {$ENDIF} ));
end;

function TNamedFoldersPanel.DeleteNamedFolder(const Name: string): boolean;
begin
  Result := TNamedFolders.DeleteNamedFolder(Name);
  if Result then
    ReloadPanelContents;
end;

destructor TNamedFoldersPanel.Destroy;
begin
  SaveLastDirectory;
  inherited;
end;

function TNamedFoldersPanel.EditNamedFolder(const Name: string): boolean;
begin
  Result := TNamedFolders.EditNamedFolder(Name);
  if Result then begin
    ReloadPanelContents;
  end;
end;

function TNamedFoldersPanel.ExecuteNamedFolder(InCurrentPanel: boolean; out Handled: boolean): boolean;
var
  PanelHandle: THandle;
  Panel: TFarPanel;
  Info: TFarPanel.TItem;
begin
  Result := False;
  Handled := True;
  if InCurrentPanel then
    PanelHandle := Self.Handle
  else
    PanelHandle := PANEL_PASSIVE;
  Panel := TFarPanel.Create(True, True);
  try
    if Panel.CurrentItemIndex >= 0 then
      if Panel.CurrentItemIndex < Panel.ItemCount then begin
        Info := Panel.Items[Panel.CurrentItemIndex];
        if (Info.FileAttributes and FILE_ATTRIBUTE_DIRECTORY) <> 0 then begin
          Result := True;
          Handled := False;
        end
        else
          Result := TNamedFolders.SetCurrentDirToNamedFolder(Info.FileName, PanelHandle);
      end;
  finally
    FreeAndNil(Panel);
  end;
end;

function TNamedFoldersPanel.ExportNamedFolderToDirectory(const Name, TargetDirectory: string; DeleteOriginal, Silent: boolean; var OverwriteState: TOverwriteState): boolean;
var
  Info: TNamedFolderInfo;
begin
  Result := False;
  if TNamedFolders.ReadInfoFromOptions(Name, Info) then
    try
      Result := ExportNamedFolderToDirectory(Info, TargetDirectory, DeleteOriginal, Silent, OverwriteState);
    finally
      FreeAndNil(Info);
    end
end;

function TNamedFoldersPanel.ExportNamedFolderToDirectory(Info: TNamedFolderInfo; const TargetDirectory: string; DeleteOriginal, Silent: boolean;  var OverwriteState: TOverwriteState): boolean;
var
  DestFN: string;
  ItemList: TStringList;
  DisplayNameStartIndex: integer;
  i: integer;
begin
  DestFN := ExpandFileName(TargetDirectory);
  ForceDirectories(DestFN);
  DestFN := IncludeTrailingPathDelimiter(DestFN) + GetLastPathElement(Info.Name);
  if Info.IsDirectory then
  begin
    Result := False;
    if TNamedFolders.ReadNamedFoldersList(Info.Name, False, ItemList, DisplayNameStartIndex) then
    begin
      Result := True;
      for i := 0 to Pred(ItemList.Count) do
      begin
        if not ExportNamedFolderToDirectory(ItemList[i], DestFN, DeleteOriginal, Silent, OverwriteState) then
          Result := False;
      end;
    end;
    if Result and DeleteOriginal then
      TNamedFolders.DeleteNamedFolder(Info.Name);
  end
  else
    Result := ExportNamedFolderToFile(Info, DestFN + NAMEDFOLDERFILE_EXTENSION, DeleteOriginal, Silent, OverwriteState);
end;

function TNamedFoldersPanel.ExportNamedFolderToFile(Info: TNamedFolderInfo; const TargetFileName: string; DeleteOriginal, Silent: boolean; var OverwriteState: TOverwriteState): boolean;
var
  Xml: TXmlDoc;
  Node: TXmlNode;
begin
  Result := False;
  Xml := TXmlDoc.Create;
  try
    Xml.SetIndentation('  ');
    Xml.Root.Name := XML_ROOTNODE;
    Node := Xml.Root.NodeNew(XML_ITEMNODE);
    if TNamedFolders.ExportNamedFolderToXml(Info, Node) then begin
      if FileExists(TargetFilename) then
        if Silent then
          Exit
        else if not AskCanOverwriteFile(TargetFileName, OverwriteState) then
          Exit;
      Xml.SaveToFile(TargetFileName);
      if DeleteOriginal then
        TNamedFolders.DeleteNamedFolder(Info.Name);
      Result := True;
    end;
  finally
    FreeAndNil(Xml);
  end;
end;

procedure TNamedFoldersPanel.FreePanelItems(const Items: PPluginPanelItemArray; const Count: TIntPtr);
begin
  inherited;
  if FileToBeSelected <> '' then
  begin
    try
      SelectFile(FileToBeSelected);
    finally
      FileToBeSelected := '';
    end;
  end;
end;

function TNamedFoldersPanel.GetCurrentDirectory: string;
begin
  Result := fDirectory;
end;

function TNamedFoldersPanel.GetFile(const Item: PPluginPanelItem; OpMode: integer; const Dir: string; Move: boolean): boolean;
begin
  if (OpMode and OPM_EDIT) <> 0 then
  begin
    Result := EditNamedFolder(CopyString( {$IFDEF FAR3_UP} Item.FileName {$ELSE} Item.FindData.cFileName {$ENDIF} ));
  end
  else
  begin
    if ExportNamedFolderToDirectory(
         CopyString( {$IFDEF FAR3_UP} Item.FileName {$ELSE} Item.FindData.cFileName {$ENDIF} ),
         Dir,
         Move,
         (OpMode and OPM_SILENT) <> 0,
         fOverwriteState
         )
    then
    begin
      Item.Flags := Item.Flags and (not PPIF_SELECTED);
      Result := True;
    end
    else
    begin
      Result := False;
    end;
  end;
end;

function TNamedFoldersPanel.GetTitle: string;
begin
  Result := fTitle;
end;

procedure TNamedFoldersPanel.GetPanelInfo(var Info: TOpenPanelInfo);
begin
  inherited;
  Info.Flags := Info.Flags
               or OPIF_RAWSELECTION
               or OPIF_SHOWNAMESONLY
               or OPIF_SHOWRIGHTALIGNNAMES
               or OPIF_SHOWPRESERVECASE
               {$IFDEF FAR3_UP}
               or OPIF_DISABLESORTGROUPS
               //or OPIF_DISABLEHIGHLIGHTING
               {$ENDIF}
               ;
  Info.StartPanelMode := Ord('6');
end;

function TNamedFoldersPanel.GetPanelItems(const Mode: TIntPtr; out Items: PPluginPanelItemArray; out Count: TIntPtr): boolean;
begin
  ValidatePanelItems;
  Count := Self.GetPanelItemsCount;
  if Count > 0 then
    Items := @fPanelItems[0]
  else
    Items := nil;
  Result := True;
end;

function TNamedFoldersPanel.GetPanelItemsCount: integer;
begin
  ValidatePanelItems;
  Result := Length(fPanelItems);
end;

function TNamedFoldersPanel.ImportNamedFolderFile(const FileName, SourceDirectory, DestinationDirectory: string; DeleteOriginal, Silent: boolean; var OverwriteState: TOverwriteState): boolean;
var
  SrcFN, DestFN: string;
  Xml: TXmlDoc;
  Node: TXmlNode;
  Info, OldInfo: TNamedFolderInfo;
  CanImport: boolean;
  SR: TSearchRec;
  i: Integer;
begin
  Result := False;
  try
    SrcFN := IncludeTrailingPathDelimiter(SourceDirectory) + FileName;
    // If source is a directory, read it as a tree of individual files
    if DirectoryExists(SrcFN) then
    begin
      if DestinationDirectory = '' then
        DestFN := FileName
      else
        DestFN := DestinationDirectory + '/' + FileName;
      if TNamedFolders.ReadInfoFromOptions(DestFN, Info, False) then
      begin
        if Info.IsDirectory then
          Result := True;
      end
      else
        Result := TNamedFolders.MakeDirectory(DestFN);
      if not Result then
        ShowWarning([GetMsg(MFailedToCreateDir), DestFN])
      else
        if FindFirst(SrcFN + '\*.*', faAnyFile, SR) = 0 then
          try
            repeat
              if (SR.Name <> '.') and (SR.Name <> '..') then
              begin
                if not ImportNamedFolderFile(SR.Name, SrcFN, DestFN, DeleteOriginal, Silent, OverwriteState) then
                  Result := False;
              end;
            until FindNext(SR) <> 0;
          finally
            FindClose(SR);
          end;
    end
    // If source is not a directory, import it from XML
    else
    begin
      Xml := TXmlDoc.Create;
      try
        Xml.LoadFromFile(SrcFN);
        if Xml.Root.Name <> XML_ROOTNODE then begin
          ShowWarning(GetMsg(MInvalidRootNode));
        end
        else begin
          for i := 0 to Pred(Xml.Root.NodeCount) do begin
            Node := Xml.Root.Nodes[i];
            if Node.ElementType = xeNormal then begin
              if Node.Name = XML_ITEMNODE then begin
                if TNamedFolders.ImportNamedFolderFromXml(Node, Info) then begin
                  try
                    if DestinationDirectory <> '' then
                      Info.Name := DestinationDirectory + '/' + Info.Name;
                    CanImport := True;
                    if TNamedFolders.ReadInfoFromOptions(Info.Name, OldInfo, False) then
                      try
                        if Silent then
                          CanImport := False
                        else if OldInfo.IsDirectory then
                          CanImport := False
                        else if not AskCanOverwriteNamedFolder(OldInfo, Info, OverwriteState) then
                          CanImport := False;
                      finally
                        FreeAndNil(OldInfo);
                      end;
                    if CanImport then begin
                      TNamedFolders.SaveInfoToOptions(Info);
                      Result := True;
                    end;
                  finally
                    FreeAndNil(Info);
                  end;
                end;
              end;
            end;
          end;
        end;
      finally
        FreeAndNil(Xml);
      end;
    end;
  finally
    if Result and (DestinationDirectory = Self.CurrentDirectory) then
      ReloadPanelContents;
  end;
end;

function TNamedFoldersPanel.MakeDirectory(const Name: string): boolean;
var
  DirName: string;
begin
  {$IFDEF DEBUG_LOGS}
  Log('Directory = %s', [Name]);
  {$ENDIF}
  Result := False;
  fPanelItemsValid := False;
  DirName := Name;
  if TMakeDirDialog.ExecuteDialog(DirName) then
    Result := TNamedFolders.MakeDirectory(GetAbsoluteDirectory(Self.CurrentDirectory, DirName))
  else
    Abort;
end;

function TNamedFoldersPanel.NewNamedFolder: boolean;
begin
  Result := TNamedFolders.NewNamedFolder(Self.CurrentDirectory, PANEL_PASSIVE);
  if Result then begin
    ReloadPanelContents;
  end;
end;

procedure TNamedFoldersPanel.OpenLastDirectory;
var
  Dir: string;
  Options: TOptions;
begin
  Options := TOptions.Create;
  try
    if Options.SaveLastPanelDirectory then
      Dir := Options.LastPanelDirectory
    else
      Dir := '';
    if (Dir <> '') then
      SetDirectory(Dir)
    else
      SetDirectory('/');
  finally
    FreeAndNil(Options);
  end;
end;

function TNamedFoldersPanel.ProcessEvent(Event: integer; Param: Pointer): TIntPtr;
var
  FileName: string;
begin
  Result := 0;
  if Event = FE_REDRAW then
  begin
    if FileToBeSelected <> '' then
    begin
      FileName := FileToBeSelected;
      FileToBeSelected := '';
      SelectFile(FileName);
    end;
  end;
end;

function TNamedFoldersPanel.ProcessKeyEvent(Key: integer; ControlState: DWORD): boolean;
const
  KeyModifiers_CTRL  = {$IFDEF FAR3_UP} LEFT_CTRL_PRESSED or RIGHT_CTRL_PRESSED {$ELSE} PKF_CONTROL {$ENDIF} ;
  KeyModifiers_ALT   = {$IFDEF FAR3_UP} LEFT_ALT_PRESSED  or RIGHT_ALT_PRESSED  {$ELSE} PKF_ALT     {$ENDIF} ;
  KeyModifiers_SHIFT = {$IFDEF FAR3_UP} SHIFT_PRESSED                           {$ELSE} PKF_SHIFT   {$ENDIF} ;
  KeyModifiers = KeyModifiers_CTRL or KeyModifiers_ALT or KeyModifiers_SHIFT;
var
  Handled: boolean;
begin
  Result := False;
  if (Key = VK_RETURN) and ((ControlState and KeyModifiers) = 0) then
  begin
    Result := True;
    if ExecuteNamedFolder(True, Handled) then
      if not Handled then
        Result := False;
  end
  else if (Key = VK_RETURN) and ((ControlState and KeyModifiers_SHIFT) <> 0) and ((ControlState and (KeyModifiers and (not KeyModifiers_SHIFT))) = 0) then
  begin
    Result := True;
    ExecuteNamedFolder(False, Handled);
  end
  else if (Key = VK_F4) and ((ControlState and KeyModifiers) = KeyModifiers_SHIFT) then
  begin
    Result := True;
    NewNamedFolder;
  end;
end;

function TNamedFoldersPanel.PutFile(const Item: PPluginPanelItem; OpMode: integer; const Dir: string; Move: boolean): TFarPanelManaged.TPutFileResult;
begin
  if ImportNamedFolderFile(
       CopyString( {$IFDEF FAR3_UP} Item.FileName {$ELSE} Item.FindData.cFileName {$ENDIF} ),
       Dir,
       CurrentDirectory,
       Move,
       (OpMode and OPM_SILENT) <> 0,
       fOverwriteState
       )
  then
  begin
    Item.Flags := Item.Flags and (not PPIF_SELECTED);
    Result := pfrOKDontMoveCursor;
  end
  else
  begin
    Result := pfrFailed;
  end;
end;

procedure TNamedFoldersPanel.ReloadPanelContents;
begin
  fPanelItemsValid := False;
  FarApi.Control(Handle, FCTL_UPDATEPANEL, 1, nil);
  FarApi.Control(Handle, FCTL_REDRAWPANEL, 0, nil);
end;

procedure TNamedFoldersPanel.SaveLastDirectory;
var
  Options: TOptions;
begin
  Options := TOptions.Create;
  try
    if Options.SaveLastPanelDirectory then begin
      Options.LastPanelDirectory := fDirectory;
      Options.Save;
    end;
  finally
    FreeAndNil(Options);
  end;
end;

function TNamedFoldersPanel.SelectFile(const Name: string): boolean;
var
  RedrawInfo: TPanelRedrawInfo;
  i: integer;
  NormalizedName: string;
  Panel: TFarPanel;
begin
  Result := False;
  FillChar(RedrawInfo, Sizeof(RedrawInfo), 0);
  {$IFDEF FAR3_UP}
  RedrawInfo.StructSize := Sizeof(RedrawInfo);
  {$ENDIF}
  NormalizedName := TNamedFolderInfo.NormalizeName(Name);
  Panel := TFarPanel.Create(Handle);
  try
    for i := 0 to Pred(Panel.ItemCount) do begin
      if AnsiCompareStr(NormalizedName, Panel.Items[i].FileName) = 0 then begin
        RedrawInfo.CurrentItem := i;
        RedrawInfo.TopPanelItem := i;
        Result := FarApi.Control(Handle, FCTL_REDRAWPANEL, 0, @RedrawInfo) <> 0;
        Break;
      end;
    end;
  finally
    FreeAndNil(Panel);
  end;
end;

procedure TNamedFoldersPanel.SetDirectory(const Value: string);
begin
  //Log('TNamedFoldersPanel.SetDirectory("%s")', [Value]);
  fDirectory := GetAbsoluteDirectory(Self.CurrentDirectory, Value);
  //Log('- FDirectory = "%s"', [fDirectory]);
  fTitle := GetMsg(MTitle) + ': ' + fDirectory;
  fPanelItemsValid := False;
  SaveLastDirectory;
end;

procedure TNamedFoldersPanel.ValidatePanelItems;
var
  Options: TOptions;
  NameList: TStringList;
  s, FileName, DisplayFileName: string;
  DisplayNameStartIndex: integer;
  Info: TNamedFolderInfo;
  i, n: Integer;
begin
  if not fPanelItemsValid then begin
    //Log('TNamedFoldersPanel.ValidatePanelItems');
    //Log('- current directory "%s"', [Self.Directory]);
    if TNamedFolders.ReadNamedFoldersList(Self.CurrentDirectory, False, NameList, DisplayNameStartIndex) then
      try
        Options := TOptions.Create;
        try
          // Add the link to parent directory, if required
          if Self.CurrentDirectory <> '' then begin
            //Log('- adding parent directory');
            SetLength(fPanelItemsHelp, NameList.Count + 1);
            fPanelItemsHelp[0].FileName := '..';
            fPanelItemsHelp[0].PanelFileName := '..';
            fPanelItemsHelp[0].Description := '';
            fPanelItemsHelp[0].IsDirectory := True;
            n := 1;
          end
          else begin
            SetLength(fPanelItemsHelp, NameList.Count);
            n := 0;
          end;
          // Read info for the found names
          for i := 0 to Pred(NameList.Count) do begin
            FileName := NameList[i];
            //Log('- loading info for "%s"', [FileName]);
            s := Copy(FileName, DisplayNameStartIndex, MaxInt);
            if ExtractPathElement(s, DisplayFileName) then
              if Options.LoadNamedFolderInfo(FileName, Info) then
                try
                  //Log('- adding "%s"', [FileName]);
                  fPanelItemsHelp[n].FileName := FileName;
                  fPanelItemsHelp[n].PanelFileName := DisplayFileName;
                  fPanelItemsHelp[n].Description := Info.DisplayableDescription;
                  fPanelItemsHelp[n].IsDirectory := Info.IsDirectory;
                  Inc(n);
                finally
                  FreeAndNil(Info);
                end;
          end;
          SetLength(fPanelItemsHelp, n);
          // Step two: Prepare panel item list
          SetLength(fPanelItems, n);
          for i := 0 to Pred(n) do begin
            FillChar(fPanelItems[i], Sizeof(fPanelItems[i]), 0);
            fPanelItems[i]. {$IFDEF FAR3_UP} FileName {$ELSE} FindData.cFileName {$ENDIF} := FarString(fPanelItemsHelp[i].FileName);
            fPanelItems[i].Description := FarString(fPanelItemsHelp[i].Description);
            fPanelItems[i].Flags := {$IFDEF FAR3_UP} PPIF_NONE {$ELSE} 0 {$ENDIF} ;
            if fPanelItemsHelp[i].IsDirectory then
              fPanelItems[i]. {$IFDEF FAR3_UP} FileAttributes {$ELSE} FindData.dwFileAttributes {$ENDIF} := FILE_ATTRIBUTE_DIRECTORY
            else
              fPanelItems[i]. {$IFDEF FAR3_UP} FileAttributes {$ELSE} FindData.dwFileAttributes {$ENDIF} := FILE_ATTRIBUTE_NORMAL;
          end;
        finally
          FreeAndNil(Options);
        end;
      finally
        FreeAndNil(NameList);
      end;
    fPanelItemsValid := True;
    //Log('TNamedFoldersPanel.ValidatePanelItems -> END');
  end;
end;

end.
